<?php
/**
* 2007-2023 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author    PrestaShop SA <contact@prestashop.com>
*  @copyright 2007-2023 PrestaShop SA
*  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/

 require_once _PS_MODULE_DIR_.'hsmsmodule/vendor/autoload.php';
 require_once(_PS_ROOT_DIR_ . '/config/config.inc.php');

 use HostedSms\WebService\HostedSmsWebService;

if (!defined('_PS_VERSION_')) {
    exit;
}

class HSMSmodule extends Module
{
    protected $config_form = false;
    protected $senders;
    protected $orderStates;
    private $defaultMessage = 'Czesc {customer_first_name} {customer_last_name}, status zamowienia o numerze {number}, zostal zmieniony.';

    public function __construct()
    {
        $this->name = 'hsmsmodule';
        $this->tab = 'administration';
        $this->version = '8.1.1';
        $this->author = 'HSMS';
        $this->need_instance = 1;

        /**
         * Set $this->bootstrap to true if your module is compliant with bootstrap (PrestaShop 1.6)
         */
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->l('API SMS dla Prestashop - integracja z HostedSMS.pl');
        $this->description = $this->l('Module to send SMS with HostedSMS');

        $this->confirmUninstall = $this->l('Are you sure you want to uninstall?');

        $this->ps_versions_compliancy = array('min' => '8.1', 'max' => _PS_VERSION_);
    }

    /**
     * Don't forget to create update methods if needed:
     * http://doc.prestashop.com/display/PS16/Enabling+the+Auto-Update
     */
    public function install()
    {
        Configuration::updateValue('HSMSMODULE_LIVE_MODE', true);
        Configuration::updateValue('HSMSMODULE_MESSAGE_STATUS_1', $this->l($this->defaultMessage));
        Configuration::updateValue('HSMSMODULE_MESSAGE_STATUS_2', $this->l($this->defaultMessage));
        Configuration::updateValue('HSMSMODULE_MESSAGE_STATUS_3', $this->l($this->defaultMessage));
        Configuration::updateValue('HSMSMODULE_MESSAGE_STATUS_4', $this->l($this->defaultMessage));

        include(dirname(__FILE__).'/sql/install.php');

        return parent::install() && $this->registerHook('actionOrderStatusUpdate') &&
                                    $this->registerHook('header') &&
                                    $this->registerHook('displayBackOfficeHeader');
    }

    public function uninstall()
    {
        Configuration::deleteByName('HSMSMODULE_LIVE_MODE');
        Configuration::deleteByName('HSMSMODULE_ACCOUNT_EMAIL');
        Configuration::deleteByName('HSMSMODULE_ACCOUNT_PASSWORD');
        Configuration::deleteByName('HSMSMODULE_SENDER');
        Configuration::deleteByName('HSMSMODULE_STATUS_1');
        Configuration::deleteByName('HSMSMODULE_STATUS_2');
        Configuration::deleteByName('HSMSMODULE_STATUS_3');
        Configuration::deleteByName('HSMSMODULE_STATUS_4');
        Configuration::deleteByName('HSMSMODULE_MESSAGE_STATUS_1');
        Configuration::deleteByName('HSMSMODULE_MESSAGE_STATUS_2');
        Configuration::deleteByName('HSMSMODULE_MESSAGE_STATUS_3');
        Configuration::deleteByName('HSMSMODULE_MESSAGE_STATUS_4');
        Configuration::deleteByName('HSMSMODULE_CHECKBOX_STATUS_1');
        Configuration::deleteByName('HSMSMODULE_CHECKBOX_STATUS_2');
        Configuration::deleteByName('HSMSMODULE_CHECKBOX_STATUS_3'); 
        Configuration::deleteByName('HSMSMODULE_CHECKBOX_STATUS_4');

        include(dirname(__FILE__).'/sql/uninstall.php');

        return parent::uninstall();
    }

    /**
     * Load the configuration form
     */
    public function getContent()
    {
        $this->orderStates = OrderState::getOrderStates((int)Context::getContext()->language->id);

        $response = $this->getValidSenders();
        if($response != null)
            $this->senders = $response->senders;
        else 
            $this->senders = array();
        /**
         * If values have been submitted in the form, process.
         */
        if (((bool)Tools::isSubmit('submitHSMSmoduleModule')) == true)
        {
            $this->postProcess();
        }

        $this->context->smarty->assign('module_dir', $this->_path);

        $output = $this->context->smarty->fetch($this->local_path.'views/templates/admin/configure.tpl');

        return $output.$this->renderForm();
    }

    /**
     * Create the form that will be displayed in the configuration of your module.
     */
    protected function renderForm()
    {
        $helper = new HelperForm();

        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->module = $this;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);

        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitHSMSmoduleModule';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false)
            .'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');

        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFormValues(), /* Add values for your inputs */
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        );

        return $helper->generateForm(array($this->getConfigForm(), $this->getSMSForm()));
    }

    protected function getConfigForm()
    {
        return array(
            'form' => array(
                'legend' => array(
                'title' => $this->l('Login'),
                'icon' => 'icon-user',
                ),
                'input' => array(
                    array(
                        'col' => 4,
                        'type' => 'text',
                        'prefix' => '<i class="icon icon-envelope"></i>',
                        'desc' => $this->l('Enter a valid email address'),
                        'name' => 'HSMSMODULE_ACCOUNT_EMAIL',
                        'label' => $this->l('Email'),
                        'required' => true,
                    ),
                    array(
                        'col' => 4,
                        'type' => 'password',
                        'prefix' => '<i class="icon icon-lock"></i>',
                        'name' => 'HSMSMODULE_ACCOUNT_PASSWORD',
                        'label' => $this->l('Password'),
                        'required' => true,
                    ),
                ),
                'submit' => array(
                    'title' => $this->l('Login'),

                ),
            ),
        );
    }

    protected function getSMSForm()
    {
        return array(
            'form' => array(
                'legend' => array(
                'title' => $this->l('SMS'),
                'icon' => 'icon-mobile',
                ),
                'input' => array(
                    array(
                        'type' => 'switch',
                        'label' => $this->l('SMS notifications'),
                        'name' => 'HSMSMODULE_LIVE_MODE',
                        'is_bool' => true,
                        'desc' => $this->l('Activate to enable SMS notifications'),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),
                    array(
                        'type' => 'select',
                        'name' => 'HSMSMODULE_SENDER',
                        'label' => $this->l('Sender'),
                        'options' => array(
                            'query' => $this->convertSendersToOptions(), 
                            'id' => 'value',
                            'name' => 'name',
                        )
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Activate to enable SMS notifications for orders in status 1'),
                        'name' => 'HSMSMODULE_CHECKBOX_STATUS_1',
                        'is_bool' => true,
                        'desc' => $this->l('Activate to enable SMS notifications for orders in status 1'),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),
                    array(
                        'type' => 'select',
                        'name' => 'HSMSMODULE_STATUS_1',
                        'label' => $this->l('Status 1'),
                        'options' => array(
                            'query' => $this->convertOrderStateToOptions(), 
                            'id' => 'value',
                            'name' => 'name',
                        )
                    ),
                    array(
                        'type' => 'textarea',
                        'name' => 'HSMSMODULE_MESSAGE_STATUS_1',
                        'label' => $this->l('Message for orders in status 1'),
                        'maxlength' => 250,
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Activate to enable SMS notifications for orders in status 2'),
                        'name' => 'HSMSMODULE_CHECKBOX_STATUS_2',
                        'is_bool' => true,
                        'desc' => $this->l('Activate to enable SMS notifications for orders in status 2'),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),
                    array(
                        'type' => 'select',
                        'name' => 'HSMSMODULE_STATUS_2',
                        'label' => $this->l('Status 2'),
                        'options' => array(
                            'query' => $this->convertOrderStateToOptions(), 
                            'id' => 'value',
                            'name' => 'name',
                        )
                    ),
                    array(
                        'type' => 'textarea',
                        'name' => 'HSMSMODULE_MESSAGE_STATUS_2',
                        'label' => $this->l('Message for orders in status 2'),
                        'maxlength' => 250,
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Activate to enable SMS notifications for orders in status 3'),
                        'name' => 'HSMSMODULE_CHECKBOX_STATUS_3',
                        'is_bool' => true,
                        'desc' => $this->l('Activate to enable SMS notifications for orders in status 3'),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),
                    array(
                        'type' => 'select',
                        'name' => 'HSMSMODULE_STATUS_3',
                        'label' => $this->l('Status 3'),
                        'options' => array(
                            'query' => $this->convertOrderStateToOptions(), 
                            'id' => 'value',
                            'name' => 'name',
                        )
                    ),
                    array(
                        'type' => 'textarea',
                        'name' => 'HSMSMODULE_MESSAGE_STATUS_3',
                        'label' => $this->l('Message for orders in status 3'),
                        'maxlength' => 250,
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Activate to enable SMS notifications for orders in status 4'),
                        'name' => 'HSMSMODULE_CHECKBOX_STATUS_4',
                        'is_bool' => true,
                        'desc' => $this->l('Activate to enable SMS notifications for orders in status 4'),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),
                    array(
                        'type' => 'select',
                        'name' => 'HSMSMODULE_STATUS_4',
                        'label' => $this->l('Status 4'),
                        'options' => array(
                            'query' => $this->convertOrderStateToOptions(), 
                            'id' => 'value',
                            'name' => 'name',
                        )
                    ),
                    array(
                        'type' => 'textarea',
                        'name' => 'HSMSMODULE_MESSAGE_STATUS_4',
                        'label' => $this->l('Message for orders in status 4'),
                        'maxlength' => 250,
                    ),
                ),
                'submit' => array(
                    'title' => $this->l('Save'),
                ),
            ),
        );
    }

    /**
     * Set values for the inputs.
     */
    protected function getConfigFormValues()
    {
        return array(
            'HSMSMODULE_LIVE_MODE' => Configuration::get('HSMSMODULE_LIVE_MODE', true),
            'HSMSMODULE_ACCOUNT_EMAIL' => Configuration::get('HSMSMODULE_ACCOUNT_EMAIL', 'contact@prestashop.com'),
            'HSMSMODULE_ACCOUNT_PASSWORD' => Configuration::get('HSMSMODULE_ACCOUNT_PASSWORD', null),
            'HSMSMODULE_SENDER' => Configuration::get('HSMSMODULE_SENDER', $this->senders != null ? $this->senders[0] : null),
            'HSMSMODULE_MESSAGE_STATUS_1' => Configuration::get('HSMSMODULE_MESSAGE_STATUS_1', null),
            'HSMSMODULE_MESSAGE_STATUS_2' => Configuration::get('HSMSMODULE_MESSAGE_STATUS_2', null),
            'HSMSMODULE_MESSAGE_STATUS_3' => Configuration::get('HSMSMODULE_MESSAGE_STATUS_3', null),
            'HSMSMODULE_MESSAGE_STATUS_4' => Configuration::get('HSMSMODULE_MESSAGE_STATUS_4', null),
            'HSMSMODULE_CHECKBOX_STATUS_1' => Configuration::get('HSMSMODULE_CHECKBOX_STATUS_1', true),
            'HSMSMODULE_CHECKBOX_STATUS_2' => Configuration::get('HSMSMODULE_CHECKBOX_STATUS_2', true),
            'HSMSMODULE_CHECKBOX_STATUS_3' => Configuration::get('HSMSMODULE_CHECKBOX_STATUS_3', true),
            'HSMSMODULE_CHECKBOX_STATUS_4' => Configuration::get('HSMSMODULE_CHECKBOX_STATUS_4', true),
            'HSMSMODULE_STATUS_1' => Configuration::get('HSMSMODULE_STATUS_1', null),
            'HSMSMODULE_STATUS_2' => Configuration::get('HSMSMODULE_STATUS_2', null),
            'HSMSMODULE_STATUS_3' => Configuration::get('HSMSMODULE_STATUS_3', null),
            'HSMSMODULE_STATUS_4' => Configuration::get('HSMSMODULE_STATUS_4', null),
        );
    }

    /**
     * Save form data.
     */
    protected function postProcess()
    {
        $this->orderStates = OrderState::getOrderStates((int)Context::getContext()->language->id);

        $form_values = $this->getConfigFormValues();

        foreach (array_keys($form_values) as $key)
        {
            if($key == 'HSMSMODULE_ACCOUNT_PASSWORD' && strlen(Tools::getValue($key)) == 0)
                continue;

            if($key == 'HSMSMODULE_SENDER' && Tools::getValue($key) == null)
                continue;
             
            Configuration::updateValue($key, Tools::getValue($key));
        }

        if(empty(Tools::getValue('HSMSMODULE_MESSAGE_STATUS_1')))
            Configuration::updateValue('HSMSMODULE_MESSAGE_STATUS_1', $this->defaultMessage);
        if(empty(Tools::getValue('HSMSMODULE_MESSAGE_STATUS_2')))
            Configuration::updateValue('HSMSMODULE_MESSAGE_STATUS_2', $this->defaultMessage);
        if(empty(Tools::getValue('HSMSMODULE_MESSAGE_STATUS_3')))
            Configuration::updateValue('HSMSMODULE_MESSAGE_STATUS_3', $this->defaultMessage);
        if(empty(Tools::getValue('HSMSMODULE_MESSAGE_STATUS_4')))
            Configuration::updateValue('HSMSMODULE_MESSAGE_STATUS_4', $this->defaultMessage);

        $response = $this->getValidSenders();
        if($response != null)
            $this->senders = $response->senders;
        else 
            $this->senders = array();

        Configuration::updateValue('HSMSMODULE_SENDER', Tools::getValue('HSMSMODULE_SENDER'));
    }

    /**
    * Add the CSS & JavaScript files you want to be loaded in the BO.
    */
    public function hookDisplayBackOfficeHeader()
    {
        if (Tools::getValue('configure') == $this->name) {
            $this->context->controller->addJS($this->_path.'views/js/back.js');
            $this->context->controller->addCSS($this->_path.'views/css/back.css');
        }
    }

    /**
     * Add the CSS & JavaScript files you want to be added on the FO.
     */
    public function hookHeader()
    {
        $this->context->controller->addJS($this->_path.'/views/js/front.js');
        $this->context->controller->addCSS($this->_path.'/views/css/front.css');
    }

    public function hookActionOrderStatusUpdate($params)
    {
        $newOrderStatus = $params['newOrderStatus'];
        $oldOrderStatus = $params['oldOrderStatus'];

        // Check if SMS notifications are enabled
        if (Configuration::get('HSMSMODULE_LIVE_MODE'))
         {                                     
            $order = new Order ($params['id_order']);
            $customer = new Customer ($order->id_customer);
            $address = new Address ($order->id_address_delivery);
            $currency = new Currency($order->id_currency);
            $totalPaid = number_format((float)$order->total_paid, 2, '.', '') . $currency->iso_code;

            $statusId = $this->l($newOrderStatus->id);

            if($statusId == Configuration::get('HSMSMODULE_STATUS_1') && Configuration::get('HSMSMODULE_CHECKBOX_STATUS_1'))
            {
                $message = Configuration::get('HSMSMODULE_MESSAGE_STATUS_1');
                if($message == NULL || strlen($message) == 0)
                    $message = $this->defaultMessage;
                $message = $this->processMessage($message, $params['id_order'], $address->phone, $customer->firstname, $customer->lastname, $totalPaid);
                $this->sendSMS($address->phone, $message);
            }
            else if($statusId == Configuration::get('HSMSMODULE_STATUS_2') && Configuration::get('HSMSMODULE_CHECKBOX_STATUS_2'))
            {
                $message = Configuration::get('HSMSMODULE_MESSAGE_STATUS_2');
                if($message == NULL || strlen($message) == 0)
                    $message = $this->defaultMessage;
                $message = $this->processMessage($message, $params['id_order'], $address->phone, $customer->firstname, $customer->lastname, $totalPaid);
                $this->sendSMS($address->phone, $message);
            }
            else if($statusId == Configuration::get('HSMSMODULE_STATUS_3') && Configuration::get('HSMSMODULE_CHECKBOX_STATUS_3'))
            {
                $message = Configuration::get('HSMSMODULE_MESSAGE_STATUS_3');
                if($message == NULL || strlen($message) == 0)
                    $message = $this->defaultMessage;
                $message = $this->processMessage($message, $params['id_order'], $address->phone, $customer->firstname, $customer->lastname, $totalPaid);
                $this->sendSMS($address->phone, $message);
            }
            else if($statusId == Configuration::get('HSMSMODULE_STATUS_4') && Configuration::get('HSMSMODULE_CHECKBOX_STATUS_4'))
            {
                $message = Configuration::get('HSMSMODULE_MESSAGE_STATUS_4');
                if($message == NULL || strlen($message) == 0)
                    $message = $this->defaultMessage;
                $message = $this->processMessage($message, $params['id_order'], $address->phone, $customer->firstname, $customer->lastname, $totalPaid);
                $this->sendSMS($address->phone, $message);
            }
         }
    }
    public function sendSMS($phoneNumber, $message)
    {
        $transactionID = $phoneNumber . "-"  . date('Y-m-d H:i:s');
        $service = new HostedSmsWebService(Configuration::get('HSMSMODULE_ACCOUNT_EMAIL'), Configuration::get('HSMSMODULE_ACCOUNT_PASSWORD'));
        try
        {
            $response = $service->sendSms($phoneNumber, $message, Configuration::get('HSMSMODULE_SENDER'), $transactionID);
        }
        catch(Exception $e)
        {
            if($e->getMessage() == "Invalid Credentials or IP")
                $this->context->controller->errors[] = $this->l("Invalid email or password to HostedSMS");
            else
                $this->context->controller->errors[] = $this->l($e->getMessage());
        }
    }

    private function processMessage($message, $orderNumber, $phone, $firstName, $lastName, $totalPaid)
    {
        $parametrs = array(
            '{number}' => $orderNumber,
            '{phone}' => $phone,
            '{customer_first_name}' => $firstName,
            '{customer_last_name}' => $lastName,
            '{total_paid}' => $totalPaid
        );
        $message = str_replace(array_keys($parametrs), $parametrs, $message);
        return $message;
    }

    private function getValidSenders()
    {
        if(strlen(Configuration::get('HSMSMODULE_ACCOUNT_EMAIL') == 0 || strlen(Configuration::get('HSMSMODULE_ACCOUNT_PASSWORD')) == 0))
            return null;

        $service = new HostedSmsWebService(Configuration::get('HSMSMODULE_ACCOUNT_EMAIL'), Configuration::get('HSMSMODULE_ACCOUNT_PASSWORD'));
        $response = '';
        try
        {
            $response = $service->getValidSenders();
        }
         catch(Exception $e)
        {
            if($e->getMessage() == "Invalid Credentials or IP")
                $this->context->controller->errors[] = $this->l("Invalid email or password to HostedSMS");
            else
                $this->context->controller->errors[] = $this->l($e->getMessage());
        }
        return $response;
    }

    private function convertSendersToOptions()
    {
        if($this->senders == null)
            return null;

        $options = array();
        foreach ($this->senders as $sender)
        {
            $options[] = array(
                'value' => $sender,
                'name' => $sender,
            );
        }

        return $options;
    }

    private function convertOrderStateToOptions()
    {
        if($this->orderStates == null)
            return null;
           
        $options = array();
        foreach ($this->orderStates as $state)
        {
            $options[] = array(
                'value' => $state['id_order_state'],
                'name' => $state['name'],
                'selected' => (Configuration::get('HSMSMODULE_MESSAGE_STATUS_1') == $state['name']),
            );
        }
    
        return $options;
    }
}
